"""工具调用服务模块 - 负责执行各种工具调用"""

from abc import ABC, abstractmethod
from typing import Dict, Any, List
from loguru import logger
import httpx
import json

from ..models.schemas import ToolCallResult


class BaseTool(ABC):
    """工具基类"""
    
    @abstractmethod
    async def execute(self, params: Dict[str, Any]) -> ToolCallResult:
        """执行工具调用"""
        pass
    
    @property
    @abstractmethod
    def name(self) -> str:
        """工具名称"""
        pass


class PageNavigationTool(BaseTool):
    """页面跳转工具"""
    
    @property
    def name(self) -> str:
        return "page_navigation"
    
    async def execute(self, params: Dict[str, Any]) -> ToolCallResult:
        """执行页面跳转"""
        try:
            page_path = params.get("page_path", "")
            operation = params.get("operation", "")
            
            logger.info(f"执行页面跳转: {operation} -> {page_path}")
            
            # 这里可以调用前端API或返回跳转指令
            result = {
                "action": "navigate",
                "page_path": page_path,
                "operation": operation,
                "success": True
            }
            
            return ToolCallResult(
                tool_name=self.name,
                result=result,
                success=True
            )
            
        except Exception as e:
            logger.error(f"页面跳转失败: {e}")
            return ToolCallResult(
                tool_name=self.name,
                result=None,
                success=False,
                error_message=str(e)
            )


class DatabaseQueryTool(BaseTool):
    """数据库查询工具"""
    
    def __init__(self, database_config: Dict[str, Any]):
        self.database_config = database_config
    
    @property
    def name(self) -> str:
        return "database_query"
    
    async def execute(self, params: Dict[str, Any]) -> ToolCallResult:
        """执行数据库查询"""
        try:
            query_type = params.get("query_type", "")
            filters = params.get("filters", {})
            
            logger.info(f"执行数据库查询: {query_type}")
            
            # 模拟数据库查询结果
            mock_results = await self._mock_database_query(query_type, filters)
            
            return ToolCallResult(
                tool_name=self.name,
                result=mock_results,
                success=True
            )
            
        except Exception as e:
            logger.error(f"数据库查询失败: {e}")
            return ToolCallResult(
                tool_name=self.name,
                result=None,
                success=False,
                error_message=str(e)
            )
    
    async def _mock_database_query(self, query_type: str, filters: Dict[str, Any]) -> Dict[str, Any]:
        """模拟数据库查询（实际项目中应该连接真实数据库）"""
        
        mock_data = {
            "production_status": {
                "total_orders": 150,
                "completed_orders": 128,
                "in_progress_orders": 18,
                "pending_orders": 4,
                "efficiency": 85.3
            },
            "material_inventory": {
                "total_materials": 1250,
                "low_stock_materials": 12,
                "out_of_stock_materials": 2,
                "recent_movements": [
                    {"material": "钢材A", "operation": "入库", "quantity": 100},
                    {"material": "塑料B", "operation": "出库", "quantity": 50}
                ]
            },
            "equipment_status": {
                "total_equipment": 45,
                "running": 38,
                "maintenance": 5,
                "fault": 2,
                "fault_list": ["设备001: 传感器故障", "设备025: 电机异常"]
            }
        }
        
        return mock_data.get(query_type, {"message": "未找到相关数据"})


class ReportGenerationTool(BaseTool):
    """报表生成工具"""
    
    @property
    def name(self) -> str:
        return "report_generation"
    
    async def execute(self, params: Dict[str, Any]) -> ToolCallResult:
        """执行报表生成"""
        try:
            report_type = params.get("report_type", "")
            date_range = params.get("date_range", {})
            
            logger.info(f"生成报表: {report_type}")
            
            # 模拟报表生成
            report_data = await self._generate_mock_report(report_type, date_range)
            
            return ToolCallResult(
                tool_name=self.name,
                result=report_data,
                success=True
            )
            
        except Exception as e:
            logger.error(f"报表生成失败: {e}")
            return ToolCallResult(
                tool_name=self.name,
                result=None,
                success=False,
                error_message=str(e)
            )
    
    async def _generate_mock_report(self, report_type: str, date_range: Dict[str, Any]) -> Dict[str, Any]:
        """模拟报表生成"""
        
        mock_reports = {
            "production_summary": {
                "report_name": "生产汇总报表",
                "period": "2024-01",
                "summary": {
                    "total_production": 15240,
                    "target_production": 16000,
                    "efficiency": 95.25,
                    "quality_rate": 98.5
                },
                "download_url": "/reports/production_summary_202401.pdf"
            },
            "material_consumption": {
                "report_name": "物料消耗报表",
                "period": "2024-01",
                "top_consumed": [
                    {"material": "钢材A", "consumed": 1250, "unit": "kg"},
                    {"material": "塑料B", "consumed": 850, "unit": "kg"}
                ],
                "download_url": "/reports/material_consumption_202401.pdf"
            }
        }
        
        return mock_reports.get(report_type, {"message": "报表类型不支持"})


class ExternalAPITool(BaseTool):
    """外部API调用工具"""
    
    def __init__(self, api_config: Dict[str, Any]):
        self.api_config = api_config
    
    @property
    def name(self) -> str:
        return "external_api"
    
    async def execute(self, params: Dict[str, Any]) -> ToolCallResult:
        """执行外部API调用"""
        try:
            api_endpoint = params.get("endpoint", "")
            method = params.get("method", "GET")
            payload = params.get("payload", {})
            
            logger.info(f"调用外部API: {method} {api_endpoint}")
            
            async with httpx.AsyncClient() as client:
                if method.upper() == "GET":
                    response = await client.get(api_endpoint, params=payload)
                elif method.upper() == "POST":
                    response = await client.post(api_endpoint, json=payload)
                else:
                    raise ValueError(f"不支持的HTTP方法: {method}")
                
                response.raise_for_status()
                result = response.json()
                
                return ToolCallResult(
                    tool_name=self.name,
                    result=result,
                    success=True
                )
                
        except Exception as e:
            logger.error(f"外部API调用失败: {e}")
            return ToolCallResult(
                tool_name=self.name,
                result=None,
                success=False,
                error_message=str(e)
            )


class ToolService:
    """工具服务管理类"""
    
    def __init__(self, database_config: Dict[str, Any] = None, api_config: Dict[str, Any] = None):
        self.tools: Dict[str, BaseTool] = {}
        
        # 注册默认工具
        self.register_tool(PageNavigationTool())
        self.register_tool(DatabaseQueryTool(database_config or {}))
        self.register_tool(ReportGenerationTool())
        self.register_tool(ExternalAPITool(api_config or {}))
    
    def register_tool(self, tool: BaseTool):
        """注册工具"""
        self.tools[tool.name] = tool
        logger.info(f"注册工具: {tool.name}")
    
    async def execute_tool(self, tool_name: str, params: Dict[str, Any]) -> ToolCallResult:
        """执行指定工具"""
        tool = self.tools.get(tool_name)
        if not tool:
            logger.error(f"工具不存在: {tool_name}")
            return ToolCallResult(
                tool_name=tool_name,
                result=None,
                success=False,
                error_message=f"工具 {tool_name} 不存在"
            )
        
        logger.info(f"执行工具: {tool_name}")
        return await tool.execute(params)
    
    def get_available_tools(self) -> List[str]:
        """获取可用工具列表"""
        return list(self.tools.keys())
    
    async def batch_execute_tools(self, tool_calls: List[Dict[str, Any]]) -> List[ToolCallResult]:
        """批量执行工具调用"""
        results = []
        for tool_call in tool_calls:
            tool_name = tool_call.get("tool_name", "")
            params = tool_call.get("params", {})
            result = await self.execute_tool(tool_name, params)
            results.append(result)
        return results 